""" Directed Graph package """
import types
import Set
import Multimap
import PureSet
class DigraphException(Exception):pass
class EdgeException(DigraphException):
	def __init__(self,val=None):
		self.value=val

class Digraph:
	"""Digraph(), Digraph(es)
	creates a directed graph initialized to contain all the
	edges contained as pairs in es
	Edges are ordered pairs of nodes.
	Nodes are objects that can be used as 
	keys of dictionaries.
	"""
	def __init__(self,es=()):
		self.nodeSet=Set.Set()
		self.adjFrom=Multimap.mmap()
		self.adjTo=Multimap.mmap()
		for e in es:
			self.addEdge(e)
	def nodes(self):
		"""nodes(self)
		returns a pure set of all the nodes in self
		"""
		return PureSet.PureSet(rep=self.nodeSet)
	def edges(self):
		"""edges(self)
		returns a list of all the edges in self
		"""
		s=[]
		for n in self.nodeSet:
			 s=s+self.incidentFrom(n)
		return s
	def addEdge(self,*e):
		"""addEdge(self,x[,y])
		adds the edge x or (x,y) to self
		"""
		if len(e)==2:
			pass
		elif len(e)==1:
			e=e[0]
		else: raise EdgeException()
		n0,n1=e
		self.addNode(n0)
		self.addNode(n1)
		self.adjFrom[n0]=n1
		self.adjTo[n1]=n0
		return e
	def addNode(self,n):
		"""addNode(self,n)
		adds the node n to self
		"""
		self.nodeSet.insert(n)
	def deleteNode(self,n):
		"""deleteNode(self,n)
		deletes the node n and all incident edges from self
		"""
		self.nodeSet.delete(n)
		for m in self.adjTo[n].members():
			self.deleteEdge(n,m)
		for m in self.adjFrom[n].members():
			self.deleteEdge(m,n)
	def deleteEdge(self,*e):
		"""deleteEdge(self,x[,y])
		deletes the edge x or (x,y) from self
		"""
		if len(e)==2:
			pass
		elif len(e)==1:
			e=e[0]
		else: raise EdgeException()
		n0,n1=e
		self.adjFrom.delete(n1,n0)
		self.adjTo.delete(n0,n1)
	def containsNode(self,n): 
		"""containsNode(self,n)
		returns true if self contains node n
		"""
		return self.nodeSet.contains(n)
	def containsEdge(self,*n): 
		"""containsEdge(self,x[,y])
		returns true if self contains edge x or (x,y)
		"""
		if len(n)==2: x,y=n
		else: x,y=n[0]
		s=self.adjFrom.get(x)
		if s: return s.contains(y)
		else: return 0
	def degree(self,n): 
		"""degree(self,n)
		returns the number of edges incident to or 
		from node n, (n,n) counts as 1.
		"""
		t=len(self.adjacentFrom(n))+ \
			len(self.adjacentTo(n))
		if self.containsEdge(n,n): return t-1
		else: return t
	def indegree(self,n): 
		"""indegree(self,n)
		returns the number of edges directed to node n
		"""
		return len(self.adjacentTo(n))
	def outdegree(self,n): 
		"""outdegree(self,n)
		returns the number of edges directed from node n
		"""
		return len(self.adjacentFrom(n))
	def incidentFrom(self,n):
		"""incidentFrom(self,n)
		returns a list of all edges directed from node n
		"""
		L=[]
		for m in self.adjacentFrom(n):
			L.append((n,m))
		return L
	def incidentTo(self,n):
		"""incidentTo(self,n)
		returns a list of all edges directed to node n
		"""
		L=[]
		for m in self.adjacentTo(n):
			L.append((m,n))
		return L
	def adjacentFrom(self,n):
		"""adjacentFrom(self,n)
		returns a set of all x s.t. self.containsEdge(n,x)
		"""
		return PureSet.PureSet(self.adjFrom.get(n,PureSet.emptySet))
	def adjacentTo(self,n):
		"""adjacentTo(self,n)
		returns a set of all x s.t. self.containsEdge(x,n)
		"""
		return PureSet.PureSet(self.adjTo.get(n,PureSet.emptySet))
	def adjacent(self,n):
		"""adjacent(self,n)
		returns a set of all x s.t. self.containsEdge(x,n) or
		self.containsEdge(n,x)
		"""
		return self.adjacentTo(n)|self.adjacentFrom(n)
	def copy(self):
		"""copy(self)
		creates a copy of digraph self
		"""
		g=Digraph()
		g.nodeSet=self.nodeSet
		g.adjTo=self.adjTo.copy()
		g.adjFrom=self.adjFrom.copy()
		return g
	def new(self): 
		"""new(self)
		creates a new digraph
		"""
		return Digraph()
	def __repr__(self):
		"""__repr__(self), repr(self)
		returns an expression that will create a copy of
		digraph self if all nodes have a repr value that
		can reconstruct them.
		"""
		return "Digraph.Digraph("+repr(self.edges())+")"
	def __str__(self): 
		"""__str__(self), str(self)
		returns a string representation of the digraph self
		"""
		return repr(self)
