""" Set 
s=Set()
s=Set(seq)
creates a set. If seq is provided, the elements 
of seq are the initial elements of the set. The 
value of seq can be anything that can be iterated 
over in a 'for x in seq:' statement, including 
another set."""
class SetEnumeration:
	"""object to return the elements of a set one at a time"""
	def __init__(self,s):
		self.members=s.members()
		self.set=s
	def hasMoreElements(self):
		"""returns true if the enumeration has more 
		elements to deliver. After returning false, 
		the SetEnumeration is no longer active and 
		will deliver no more elements. """
		if len(self.members)>0:
			return 1
		self.set._removeObserver(self)
		return 0
	def nextElement(self):
		"""returns the next element in the enumeration. 
		Raises IndexError if it is called when no more 
		elements are present. """
		try:
			return self.members.pop()
		except:
			self.set._removeObserver(self)
			raise IndexError
	def __getitem__(self,i):
		"""equivalent to e.nextElement(). 
		The parameter is ignored."""
		return self.nextElement()
	__nonzero__=hasMoreElements
	def _delete(self,item):
		try:
			self.members.remove(item)
		except:
			pass
	def _insert(self,item):
		self.members.insert(0,item)

class Set:
	"Create a set: Set() or Set(seq)"
	def __init__(self,stuff=()):
		self.rep={}
		self.cache=None
		self.observers=[]
		for x in stuff:
			self.insert(x)
	def _registerObserver(self,observer):
		self.observers.append(observer)
	def _removeObserver(self,observer):
		try:
			self.observers.remove(observer)
		except:
			pass
	def insert(self,x):
		"Insert into a set: s.insert(x)"
		if not self.rep.has_key(x):
			self.cache=None
			self.rep[x]=x
			for ob in self.observers:
				ob._insert(x)
	def delete(self,x):
		"delete an item: s.delete(x)"
		if self.rep.has_key(x):
			self.cache=None
			del self.rep[x]
			for ob in self.observers:
				ob._delete(x)
	def with(self,i):
		"create a copy of a set with an item added: s.with(i)"
		t=self.copy()
		t.insert(i)
		return t
	def without(self,i):
		"create a copy of a set with an item removed: s.without(i)"
		t=self.copy()
		t.delete(i)
		return t
	def contains(self,x):
		"Test for membership: s.contains(x)"
		return self.rep.has_key(x)
	__contains__=contains
	def members(self):
		"List members of set: s.members()"
		if not self.cache:
			self.cache=self.rep.keys()
		return self.cache[:]
	def items(self): 
		"same as members(): s.items()"
		return self.members()
	def elements(self):
		"""returns a SetEnumeration object that will deliver 
		the elements of set s one by one: s.elements()"""
		e=SetEnumeration(self)
		self._registerObserver(e)
		return e
	def copy(self):
		"Copy of a set: s.copy()"
		x=Set()
		x.rep=self.rep.copy()
		return x
	def new(self):
		"Create a new set like this one: s.new()"
		x=Set()
		return x
	def __or__(self,s):
		"Set Union: x|y "
		x=self.copy()
		for y in s.members():
			x.insert(y)
		return x
	def __and__(self,s):
		"Set Intersection: x&y"
		x=Set()
		for y in self:
			if s.contains(y):
				x.insert(y)
		return x
	def __sub__(self,s):
		"Relative complement: s1 - s2"
		x=Set()
		for y in self:
			if not s.contains(y):
				x.insert(y)
		return x
	def issubsetof(self,other):
		"""returns true (1) if all elements of s are 
		also in t, otherwise false (0): s.issubsetof(t)"""
		if len(self)>len(other):
			return 0
		for x in self:
			if not other.contains(x):
				return 0
		return 1
	def ispropersubsetof(self,other):
		"""returns true (1) if all elements of s are 
		also in t but there are one or more elements 
		of t that are not in s, otherwise false (0):
		s.ispropersubsetof(t)"""
		if len(self)>=len(other):
			return 0
		for x in self:
			if not other.contains(x):
				return 0
		return 1
	def equals(self,other):
		"""s.equals(t): returns true if all elements 
		of s are in t and all elements of t are in s."""
		if len(self)!=len(other):
			return 0
		for x in self:
			if not other.contains(x):
				return 0
		return 1
	def __cmp__(self,other):
		"""returns 
			-1 if self is a proper subset of other
			0 if self == other
			1 otherwise
			Note: not symmetric """
		d = len(self)-len(other)
		if d>0: return 1
		for x in self:
			if not other.contains(x):
				return 1
		if d==0: return 0
		else: return -1
	def __getitem__(self,i):
		"Get i-th item, no particular order: s[i]"
		if self.cache is None:
			self.cache=self.rep.keys()
		return self.cache[i]
	def arb(self):
		"returns an arbitrary element of s: s.arb()"
		if self:
			return self[0]
		else:
			import SetExceptions
			raise SetExceptions.EmptySetException()
	def __len__(self):
		"returns the number of elements in s."
		return len(self.rep)
	def __nonzero__(self):
		"equivalent to len(s)>0"
		return len(self.rep)!=0
	def __str__(self):
		"""converts s into a string of the form 
		{ x0, x1, ..., xn-1 } for elements xi."""
		return "{"+str(self.rep.keys())[1:-1]+"}"
	def __repr__(self):
		"""converts s into a string of the form 
		Set(( x0, x1, ..., xn-1 ))
		for elements xi."""
		s="Set(("
		for x in self.rep.keys() :
			s=s+repr(x)+","
		return s+"))"
